########################################################################################
#This script was written for the biogeographic analyses of Carbonaro et al. 2018
#All files can be found at GitHub repository: https://figshare.com/s/6b42cf2d4d0cadde7e11
#We conducted ancestral area reconstruction and biogeographic stochastic mapping (BSM) 
#using the package BioGeoBEARS (Matzke, 2013) 
#correspondece to: fabiocarbonaro@gmail.com or gsferreirabio@gmail.com
########################################################################################

# Loading necessary packages
library(GenSA)
library(FD)       
library(snow)   
library(parallel)
library(BioGeoBEARS)
library(roxygen2)
library(optimx) 

#check version of optmix; if >2014 then OK
sessionInfo()
source("http://phylo.wdfiles.com/local--files/biogeobears/BioGeoBEARS_add_fossils_randomly_v1.R")
source("http://phylo.wdfiles.com/local--files/biogeobears/BioGeoBEARS_basics_v1.R")
source("http://phylo.wdfiles.com/local--files/biogeobears/BioGeoBEARS_calc_transition_matrices_v1.R")
source("http://phylo.wdfiles.com/local--files/biogeobears/BioGeoBEARS_classes_v1.R")
source("http://phylo.wdfiles.com/local--files/biogeobears/BioGeoBEARS_detection_v1.R")
source("http://phylo.wdfiles.com/local--files/biogeobears/BioGeoBEARS_DNA_cladogenesis_sim_v1.R")
source("http://phylo.wdfiles.com/local--files/biogeobears/BioGeoBEARS_extract_Qmat_COOmat_v1.R")
source("http://phylo.wdfiles.com/local--files/biogeobears/BioGeoBEARS_generics_v1.R")
source("http://phylo.wdfiles.com/local--files/biogeobears/BioGeoBEARS_models_v1.R")
source("http://phylo.wdfiles.com/local--files/biogeobears/BioGeoBEARS_on_multiple_trees_v1.R")
source("http://phylo.wdfiles.com/local--files/biogeobears/BioGeoBEARS_plots_v1.R")
source("http://phylo.wdfiles.com/local--files/biogeobears/BioGeoBEARS_readwrite_v1.R")
source("http://phylo.wdfiles.com/local--files/biogeobears/BioGeoBEARS_simulate_v1.R")
source("http://phylo.wdfiles.com/local--files/biogeobears/BioGeoBEARS_SSEsim_makePlots_v1.R")
source("http://phylo.wdfiles.com/local--files/biogeobears/BioGeoBEARS_SSEsim_v1.R")
source("http://phylo.wdfiles.com/local--files/biogeobears/BioGeoBEARS_stochastic_mapping_v1.R")
source("http://phylo.wdfiles.com/local--files/biogeobears/BioGeoBEARS_stratified_v1.R")
source("http://phylo.wdfiles.com/local--files/biogeobears/BioGeoBEARS_univ_model_v1.R")
source("http://phylo.wdfiles.com/local--files/biogeobears/calc_uppass_probs_v1.R")
source("http://phylo.wdfiles.com/local--files/biogeobears/calc_loglike_sp_v01.R")
source("http://phylo.wdfiles.com/local--files/biogeobears/get_stratified_subbranch_top_downpass_likelihoods_v1.R")
source("http://phylo.wdfiles.com/local--files/biogeobears/runBSM_v1.R")
source("http://phylo.wdfiles.com/local--files/biogeobears/stochastic_map_given_inputs.R")
source("http://phylo.wdfiles.com/local--files/biogeobears/summarize_BSM_tables_v1.R")
source("http://phylo.wdfiles.com/local--files/biogeobears/cladoRcpp.R") 
calc_loglike_sp = compiler::cmpfun(calc_loglike_sp_prebyte)
calc_independent_likelihoods_on_each_branch = compiler::cmpfun(calc_independent_likelihoods_on_each_branch_prebyte)

#Set the working directory
setwd("/carbonaro-et-al-2018/common-input-files")
getwd()

#load the saved tree into BioGeoBEARS
trfn = np("timetree.newick")

#Look again at the timetree
tr = read.tree(trfn)
tr
plot(tr)
axisPhylo()

#Load the geographic distribution file
metacryphaeus.geog = np("Metacryphaeus_biogeog.data")
moref(metacryphaeus.geog)
#Look at your geographic range data
tipranges <- getranges_from_LagrangePHYLIP(lgdata_fn=metacryphaeus.geog)
tipranges

#Set the maximum number of areas any species may occupy
#this cannot be larger than the total number of areas on the geography file
max_range_size = 3

###################################################################################
#Set the models parameters
#For this analysis we ran three nested models (standard, +w, +w+j) based on DEC
###################################################################################

###################################################################################
#Run DEC M0 (w=1; j=0)
###################################################################################
BioGeoBEARS_run_object = define_BioGeoBEARS_run()
BioGeoBEARS_run_object$force_sparse=FALSE    # sparse=FALSE causes pathology & isn't much faster at this scale
BioGeoBEARS_run_object$speedup=TRUE          # shorcuts to speed ML search; use FALSE if worried (e.g. >3 params)
BioGeoBEARS_run_object$use_optimx = "GenSA"
BioGeoBEARS_run_object$calc_ancprobs=TRUE    # get ancestral states from optim run

# Set up a time-stratified analysis 
BioGeoBEARS_run_object$timesfn = "Metacryphaeus-timeperiods.txt"
BioGeoBEARS_run_object$dispersal_multipliers_fn = "Metacryphaeus_dispersal_multipliers.txt"
BioGeoBEARS_run_object$areas_allowed_fn = "Metacryphaeus-areas-allowed.txt"

# Input the maximum range size
BioGeoBEARS_run_object$max_range_size = max_range_size

# Multicore processing if desired
BioGeoBEARS_run_object$num_cores_to_use=4

#Turn sparse matrix exponentiation off
BioGeoBEARS_run_object$force_sparse = FALSE

#Give BioGeoBEARS the location of the geography file
BioGeoBEARS_run_object$geogfn = metacryphaeus.geog

#Give BioGeoBEARS the location of the topology file
BioGeoBEARS_run_object$trfn = trfn

# Load the input files into the model object 
# (It also runs some checks on these inputs for certain errors.)
BioGeoBEARS_run_object = readfiles_BioGeoBEARS_run(BioGeoBEARS_run_object)

#Divide the tree up by timeperiods
BioGeoBEARS_run_object = section_the_tree(inputs=BioGeoBEARS_run_object, make_master_table=TRUE, plot_pieces=FALSE, cut_fossils = FALSE)
# Check the stratified tree description in this table:
BioGeoBEARS_run_object$master_table

# Default settings to get ancestral states
BioGeoBEARS_run_object$return_condlikes_table = TRUE
BioGeoBEARS_run_object$calc_TTL_loglike_from_condlikes_table = TRUE
BioGeoBEARS_run_object$calc_ancprobs = TRUE

# Set up DEC model
# Look at the BioGeoBEARS_run_object; it's just a list of settings etc.
BioGeoBEARS_run_object

# Add w as a static parameter
wstart = 1
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["w","init"] = wstart
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["w","est"] = wstart

# This contains the model object
BioGeoBEARS_run_object$BioGeoBEARS_model_object

# This table contains the parameters of the model 
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["d","min"]= 0.00000001 
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["e","min"]= 0.00000001 
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["d","max"]= 3 
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["e","max"]= 3

# Run this to check inputs. Read the error messages if you get them!
check_BioGeoBEARS_run(BioGeoBEARS_run_object)

# For a slow analysis, run once, then set runslow=FALSE to just 
# load the saved result.
#DEC M0 = w=1;j=0; standard tree
runslow = TRUE
resfn = "Metacrypheaus_DEC_M0.Rdata"
if (runslow)
{
  res = bears_optim_run(BioGeoBEARS_run_object)
  res    
  
  save(res, file=resfn)
  resDEC = res
} else {
  # Loads to "res"
  load(resfn)
  resDEC = res
}

#######################################################
# Run M1 - DEC+w (w as free parameter; j=0)
#######################################################
BioGeoBEARS_run_object = define_BioGeoBEARS_run()
BioGeoBEARS_run_object$trfn = trfn
BioGeoBEARS_run_object$geogfn = metacryphaeus.geog
BioGeoBEARS_run_object$max_range_size = max_range_size

# Set up the stratified part
BioGeoBEARS_run_object$timesfn = "Metacryphaeus-timeperiods.txt"
BioGeoBEARS_run_object$dispersal_multipliers_fn = "Metacryphaeus_dispersal_multipliers.txt"
BioGeoBEARS_run_object$areas_allowed_fn = "Metacryphaeus-areas-allowed.txt"

BioGeoBEARS_run_object$speedup=TRUE          # shorcuts to speed ML search; use FALSE if worried (e.g. >3 params)
BioGeoBEARS_run_object$use_optimx = "GenSA"
BioGeoBEARS_run_object$num_cores_to_use=4
BioGeoBEARS_run_object$force_sparse=FALSE    # sparse=FALSE causes pathology & isn't much faster at this scale
BioGeoBEARS_run_object$calc_ancprobs=TRUE    # get ancestral states from optim run

# Load the input files into the model object
# (It also runs some checks on these inputs for certain errors.)
BioGeoBEARS_run_object = readfiles_BioGeoBEARS_run(BioGeoBEARS_run_object)

# Divide the tree up by timeperiods/strata (uncomment this for stratified analysis)
BioGeoBEARS_run_object = section_the_tree(inputs=BioGeoBEARS_run_object, make_master_table=TRUE, plot_pieces=FALSE, cut_fossils = FALSE)
# The stratified tree is described in this table:
BioGeoBEARS_run_object$master_table

# Good default settings to get ancestral states
BioGeoBEARS_run_object$return_condlikes_table = TRUE
BioGeoBEARS_run_object$calc_TTL_loglike_from_condlikes_table = TRUE
BioGeoBEARS_run_object$calc_ancprobs = TRUE

# Set up DEC+w model
# Get the ML parameter values from the 2-parameter nested model
# (this will ensure that the 3-parameter model always does at least as good)
dstart = resDEC$outputs@params_table["d","est"]
estart = resDEC$outputs@params_table["e","est"]

# Input starting values for d, e  and define max and min
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["d","min"]= 0.00000001 
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["e","min"]= 0.00000001
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["d","max"]= 3 
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["e","max"]= 3
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["d","init"] = dstart
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["d","est"] = dstart
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["e","init"] = estart
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["e","est"] = estart

# Add w as a free parameter  and define max and min
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["w","type"]= "free"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["w","min"]= 0.00000001
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["w","max"]= 3

check_BioGeoBEARS_run(BioGeoBEARS_run_object)

#DEC M1 = w=free;j=0; standard tree
resfn = "Metacrypheaus_DECm1.Rdata"
runslow = TRUE
if (runslow)
{
  #sourceall("/Dropbox/_njm/__packages/BioGeoBEARS_setup/")
  
  res = bears_optim_run(BioGeoBEARS_run_object)
  res    
  
  save(res, file=resfn)
  
  resDECw = res
} else {
  # Loads to "res"
  load(resfn)
  resDECw = res
}

#######################################################
# Run M2 DEC+j+w (w and j as free parameters)
#######################################################
BioGeoBEARS_run_object = define_BioGeoBEARS_run()
BioGeoBEARS_run_object$trfn = trfn
BioGeoBEARS_run_object$geogfn = metacryphaeus.geog
BioGeoBEARS_run_object$max_range_size = max_range_size

# Set up the stratified part
BioGeoBEARS_run_object$timesfn = "Metacryphaeus-timeperiods.txt"
BioGeoBEARS_run_object$dispersal_multipliers_fn = "Metacryphaeus_dispersal_multipliers.txt"
BioGeoBEARS_run_object$areas_allowed_fn = "Metacryphaeus-areas-allowed.txt"

BioGeoBEARS_run_object$speedup=TRUE          # shorcuts to speed ML search; use FALSE if worried (e.g. >3 params)
BioGeoBEARS_run_object$use_optimx = "GenSA"
BioGeoBEARS_run_object$num_cores_to_use=4
BioGeoBEARS_run_object$force_sparse=FALSE    # sparse=FALSE causes pathology & isn't much faster at this scale
BioGeoBEARS_run_object$calc_ancprobs=TRUE    # get ancestral states from optim run

# This function loads the dispersal multiplier matrix etc. from the text files into the model object. Required for these to work!
# (It also runs some checks on these inputs for certain errors.)
BioGeoBEARS_run_object = readfiles_BioGeoBEARS_run(BioGeoBEARS_run_object)

# Divide the tree up by timeperiods/strata (uncomment this for stratified analysis)
BioGeoBEARS_run_object = section_the_tree(inputs=BioGeoBEARS_run_object, make_master_table=TRUE, plot_pieces=FALSE, cut_fossils = FALSE)
# The stratified tree is described in this table:
BioGeoBEARS_run_object$master_table

# Good default settings to get ancestral states
BioGeoBEARS_run_object$return_condlikes_table = TRUE
BioGeoBEARS_run_object$calc_TTL_loglike_from_condlikes_table = TRUE
BioGeoBEARS_run_object$calc_ancprobs = TRUE

# Set up DEC+w+j model
# Get the ML parameter values from the 2-parameter nested model
# (this will ensure that the 3-parameter model always does at least as good)
dstart = resDEC$outputs@params_table["d","est"]
estart = resDEC$outputs@params_table["e","est"]
wstart = resDEC$outputs@params_table["w","est"]

# Input starting values for d, e, and w and define max and min
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["d","min"]= 0.00000001 
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["e","min"]= 0.00000001
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["d","max"]= 3 
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["e","max"]= 3
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["d","init"] = dstart
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["d","est"] = dstart
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["e","init"] = estart
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["e","est"] = estart

# Add w as a free parameter and define max and min
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["w","type"] = "free"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["w","init"] = wstart
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["w","est"] = wstart
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["w","min"]= 0.00000001
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["w","max"]= 3

# Add j as a free parameter and define max and min
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["j","type"]= "free"
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["j","init"]= 0.00000001
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["j","est"]= 0.00000001
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["j","min"]= 0.00000001
BioGeoBEARS_run_object$BioGeoBEARS_model_object@params_table["j","max"]= 3


check_BioGeoBEARS_run(BioGeoBEARS_run_object)

#DEC M2 = w=free;j=free
resfn = "Metacryphaeus_DEC+w+j_M2.Rdata"
runslow = TRUE
if (runslow)
{
  #sourceall("/Dropbox/_njm/__packages/BioGeoBEARS_setup/")
  
  res = bears_optim_run(BioGeoBEARS_run_object)
  res    
  
  save(res, file=resfn)
  
  resDECwj = res
} else {
  # Loads to "res"
  load(resfn)
  resDECwj = res
}

#######################################################
# PDF plots
#######################################################
pdffn = "Metacrypheaus_DEC.pdf"
pdf(pdffn, width=6, height=12)

#######################################################
# Plot ancestral states - M0 DEC
#######################################################
analysis_titletxt ="DEC M0 (w=1, j=0) on Metacrypheaus"

# Setup
results_object = resDEC
scriptdir = np(system.file("extdata/a_scripts", package="BioGeoBEARS"))

# States
res2 = plot_BioGeoBEARS_results(results_object, analysis_titletxt, addl_params=list("j", "w"), plotwhat="text", label.offset=0.45, tipcex=0.7, statecex=0.7, splitcex=0.6, titlecex=0.8, plotsplits=TRUE, cornercoords_loc=scriptdir, include_null_range=TRUE, tr=tr, tipranges=tipranges)

# Pie chart
plot_BioGeoBEARS_results(results_object, analysis_titletxt, addl_params=list("j", "w"), plotwhat="pie", label.offset=0.45, tipcex=0, statecex=0.45, splitcex=0.45, titlecex=0.8, plotsplits=TRUE, cornercoords_loc=scriptdir, include_null_range=TRUE, tr=tr, tipranges=tipranges)

#######################################################
# Plot ancestral states - M1 DEC+w
#######################################################
analysis_titletxt ="DEC M1 (w=free, j=0) on Metacrypheaus"

# Setup
results_object = resDECw
scriptdir = np(system.file("extdata/a_scripts", package="BioGeoBEARS"))

# States
res3 = plot_BioGeoBEARS_results(results_object, analysis_titletxt, addl_params=list("j", "w"), plotwhat="text", label.offset=0.45, tipcex=0.7, statecex=0.7, splitcex=0.6, titlecex=0.8, plotsplits=TRUE, cornercoords_loc=scriptdir, include_null_range=TRUE, tr=tr, tipranges=tipranges)

# Pie chart
plot_BioGeoBEARS_results(results_object, analysis_titletxt, addl_params=list("j", "w"), plotwhat="pie", label.offset=0.45, tipcex=0, statecex=0.45, splitcex=0.45, titlecex=0.8, plotsplits=TRUE, cornercoords_loc=scriptdir, include_null_range=TRUE, tr=tr, tipranges=tipranges)

#######################################################
# Plot ancestral states - M2 DEC+w+j
#######################################################
analysis_titletxt ="DEC M2 (w=free, j=free) on Metacrypheaus"

# Setup
results_object = resDECwj
scriptdir = np(system.file("extdata/a_scripts", package="BioGeoBEARS"))

# States
res4 = plot_BioGeoBEARS_results(results_object, analysis_titletxt, addl_params=list("j", "w"), plotwhat="text", label.offset=0.45, tipcex=0.7, statecex=0.7, splitcex=0.6, titlecex=0.8, plotsplits=TRUE, cornercoords_loc=scriptdir, include_null_range=TRUE, tr=tr, tipranges=tipranges)

# Pie chart
plot_BioGeoBEARS_results(results_object, analysis_titletxt, addl_params=list("j", "w"), plotwhat="pie", label.offset=0.45, tipcex=0, statecex=0.45, splitcex=0.45, titlecex=0.8, plotsplits=TRUE, cornercoords_loc=scriptdir, include_null_range=TRUE, tr=tr, tipranges=tipranges)


dev.off()  # Turn off PDF
cmdstr = paste("open ", pdffn, sep="")
system(cmdstr) # Plot it


#########################################################################
#########################################################################
# 
# CALCULATE SUMMARY STATISTICS TO COMPARE
# DEC, DEC+w, DEC+w+j, DIVALIKE, DIVALIKE+w, DIVALIKE+w+j
# 
#########################################################################
#########################################################################

# Set up empty tables to hold the statistical results
restable = NULL
teststable = NULL

#######################################################
# Statistics -- DEC vs. DEC+w
#######################################################
# Extract the log-likelihood
LnL_2 = get_LnL_from_BioGeoBEARS_results_object(resDEC)
LnL_1 = get_LnL_from_BioGeoBEARS_results_object(resDECw)

numparams1 = 3
numparams2 = 2
stats = AICstats_2models(LnL_1, LnL_2, numparams1, numparams2)
stats

# DEC, null model for Likelihood Ratio Test (LRT)
res2 = extract_params_from_BioGeoBEARS_results_object(results_object=resDEC, returnwhat="table", addl_params=c("j", "w"), paramsstr_digits=4)
# DEC+w, alternative model for Likelihood Ratio Test (LRT)
res1 = extract_params_from_BioGeoBEARS_results_object(results_object=resDECw, returnwhat="table", addl_params=c("j", "w"), paramsstr_digits=4)

# The null hypothesis for a Likelihood Ratio Test (LRT) is that two models
# confer the same likelihood on the data. See: Brian O'Meara's webpage:
# http://www.brianomeara.info/tutorials/aic
# ...for an intro to LRT, AIC, and AICc

rbind(res2, res1)
tmp_tests = conditional_format_table(stats)

restable = rbind(restable, res2, res1)
teststable = rbind(teststable, tmp_tests)

#######################################################
# Statistics -- DEC+w vs. DEC+w+j
#######################################################
# Extract the log-likelihood
LnL_3 = get_LnL_from_BioGeoBEARS_results_object(resDECwj)


numparams3 = 4
stats = AICstats_2models(LnL_3, LnL_1, numparams3, numparams1)
stats

# DEC+w+J, alternative model for Likelihood Ratio Test (LRT)
res3 = extract_params_from_BioGeoBEARS_results_object(results_object=resDECwj, returnwhat="table", addl_params=c("j", "w"), paramsstr_digits=4)

# The null hypothesis for a Likelihood Ratio Test (LRT) is that two models
# confer the same likelihood on the data. See: Brian O'Meara's webpage:
# http://www.brianomeara.info/tutorials/aic
# ...for an intro to LRT, AIC, and AICc

rbind(restable, res3)
tmp_tests = conditional_format_table(stats)

restable = rbind(restable, res3)
teststable = rbind(teststable, tmp_tests)


#########################################################################
# RESULTS: DEC, DEC+w, DEC+w+j, DIVALIKE, DIVALIKE+w, DIVALIKE+w+j
#########################################################################
teststable$alt = c("DEC+w", "DEC+w+j")
teststable$null = c("DEC", "DEC+w")
row.names(restable) = c("DEC", "DEC+w", "DEC+w+j")

# Look at the results
restable
teststable

#######################################################
# Save the results tables
#######################################################

# Loads to "restable"
save(restable, file="restable_x1.Rdata")
load(file="restable_x1.Rdata")

# Loads to "teststable"
save(teststable, file="teststable_x1.Rdata")
load(file="teststable_x1.Rdata")

# Also save to text files
write.table(restable, file="restablex1.txt", quote=FALSE, sep="\t")
write.table(unlist_df(teststable), file="teststablex1.txt", quote=FALSE, sep="\t")

#######################################################
# Model weights of all three models
#######################################################
restable2 = restable

# With AICs:
AIC = calc_AIC_column(LnL_vals=restable$LnL, nparam_vals=restable$numparams)
restable = cbind(restable, AIC)
restable = AkaikeWeights_on_summary_table(restable=restable, colname_to_use="AIC")
restable

rellike_AIC = restable$AIC_wt / sum(restable$AIC_wt)
restable_AIC_rellike = cbind(restable, rellike_AIC)

# With AICcs -- factors in sample size
samplesize = length(tr$tip.label)
AIC = calc_AICc_column(LnL_vals=restable$LnL, nparam_vals=restable$numparams, samplesize=samplesize)
restable2 = cbind(restable2, AIC)
restable2 = AkaikeWeights_on_summary_table(restable=restable2, colname_to_use="AIC")
restable2
names(restable2) = c("LnL", "numparams", "d", "e", "j", "w", "AICc", "AICc_wt_vBest")
rellike_AICc = restable2$AICc_wt_vBest / sum(restable2$AICc_wt_vBest)
restable_AICc_rellike = cbind(restable2, rellike_AICc)

# Also save to text files
write.table(restable_AIC_rellike, file="restable_AIC_rellikex1.txt", quote=FALSE, sep="\t")
write.table(restable_AICc_rellike, file="restable_AICc_rellikex1.txt", quote=FALSE, sep="\t")

#########################################################################
#########################################################################
##                                                                      #
##Biogeographic Stochastic Mapping                                      #
##                                                                      #
#########################################################################
#########################################################################
model_name = "DECwj"
res = resDECwj

pdffn = paste0("Metacryphaeus_", model_name, "_DEC_M2.pdf")
pdf(pdffn, width=6, height=6)

analysis_titletxt = paste0(model_name, " on Metacryphaeus")

# Setup
results_object = res
scriptdir = np(system.file("extdata/a_scripts", package="BioGeoBEARS"))

# States
res2 = plot_BioGeoBEARS_results(results_object, analysis_titletxt, addl_params=list("w","j"), plotwhat="text", label.offset=0.45, tipcex=0.7, statecex=0.7, splitcex=0.6, titlecex=0.8, plotsplits=TRUE, cornercoords_loc=scriptdir, include_null_range=TRUE, tr=tr, tipranges=tipranges)

# Pie chart
plot_BioGeoBEARS_results(results_object, analysis_titletxt, addl_params=list("w","j"), plotwhat="pie", label.offset=0.45, tipcex=0.7, statecex=0.7, splitcex=0.6, titlecex=0.8, plotsplits=TRUE, cornercoords_loc=scriptdir, include_null_range=TRUE, tr=tr, tipranges=tipranges)

dev.off()  # Turn off PDF
cmdstr = paste("open ", pdffn, sep="")
system(cmdstr) # Plot it

#######################################################
# Stochastic mapping on DEC M2 stratified 
#######################################################
clado_events_tables = NULL
ana_events_tables = NULL
lnum = 0

#######################################################
# Get the inputs for Biogeographical Stochastic Mapping
#######################################################
BSM_inputs_fn = "BSM_inputs_file-2.Rdata"
runInputsSlow = TRUE
if (runInputsSlow)
{
  stochastic_mapping_inputs_list = get_inputs_for_stochastic_mapping(res=res)
  save(stochastic_mapping_inputs_list, file=BSM_inputs_fn)
} else {
  # Loads to "stochastic_mapping_inputs_list"
  load(BSM_inputs_fn)
} # END if (runInputsSlow)

# Check inputs
names(stochastic_mapping_inputs_list)
stochastic_mapping_inputs_list$phy2
stochastic_mapping_inputs_list$COO_weights_columnar
stochastic_mapping_inputs_list$unconstr
set.seed(seed=as.numeric(Sys.time()))

runBSMslow = TRUE
if (runBSMslow == TRUE)
{
  # Saves to: RES_clado_events_tables.Rdata
  # Saves to: RES_ana_events_tables.Rdata
  BSM_output = runBSM(res, stochastic_mapping_inputs_list=stochastic_mapping_inputs_list, maxnum_maps_to_try=100, nummaps_goal=100, maxtries_per_branch=40000, save_after_every_try=TRUE, savedir=getwd(), seedval=12345, wait_before_save=0.01)
  
  RES_clado_events_tables = BSM_output$RES_clado_events_tables
  RES_ana_events_tables = BSM_output$RES_ana_events_tables
} else {
  # Load previously saved...
  
  # Loads to: RES_clado_events_tables
  load(file="RES_clado_events_tables.Rdata")
  # Loads to: RES_ana_events_tables
  load(file="RES_ana_events_tables.Rdata")
  BSM_output = NULL
  BSM_output$RES_clado_events_tables = RES_clado_events_tables
  BSM_output$RES_ana_events_tables = RES_ana_events_tables
} # END if (runBSMslow == TRUE)

# Extract BSM output
clado_events_tables = BSM_output$RES_clado_events_tables
ana_events_tables = BSM_output$RES_ana_events_tables
head(clado_events_tables[[1]])
head(ana_events_tables[[1]])
length(clado_events_tables)
length(ana_events_tables)

include_null_range = TRUE
areanames = names(tipranges@df)
areas = areanames
max_range_size = 4

states_list_0based = rcpp_areas_list_to_states_list(areas=areas, maxareas=max_range_size, include_null_range=include_null_range)

colors_list_for_states = get_colors_for_states_list_0based(areanames=areanames, states_list_0based=states_list_0based, max_range_size=max_range_size, plot_null_range=TRUE)

############################################
# Setup for painting a single stochastic map
############################################
scriptdir = np(system.file("extdata/a_scripts", package="BioGeoBEARS"))
stratified = TRUE
clado_events_table = clado_events_tables[[1]]
ana_events_table = ana_events_tables[[1]]

############################################
# Open a PDF
############################################
pdffn = paste0(model_name, "_single_stochastic_map.pdf")
pdf(file=pdffn, width=6, height=6)

# Convert the BSM into a modified res object
master_table_cladogenetic_events = clado_events_tables[[1]]
resmod = stochastic_map_states_into_res(res=res, master_table_cladogenetic_events=master_table_cladogenetic_events, stratified=stratified)

#We had to manually insert all colors because their number didn't match on both lists
colors_list_for_states<-c("#000000", "#0000FF", "#00FFFF", "#00CD00", "#FFFF00", "#FF0000", "#0080FF", "#006680", "#808080",
                          "#800080", "#00E680", "#80FF80", "#808080", "#80E600", "#806600", "#FF8000", "#0099AA", 
                          "#55AAAA", "#5555AA", "#559955", "#554455", "#AA5555", "#55EE55", "#559955", "#AAAA55",
                          "#AA9900")

plot_BioGeoBEARS_results(results_object=resmod, analysis_titletxt="Stochastic map", addl_params=list("j"), label.offset=0.5, plotwhat="text", cornercoords_loc=scriptdir, root.edge=TRUE, colors_list_for_states=colors_list_for_states, skiptree=FALSE, show.tip.label=TRUE)

# Paint on the branch states
paint_stochastic_map_branches(res=resmod, master_table_cladogenetic_events=master_table_cladogenetic_events, colors_list_for_states=colors_list_for_states, lwd=5, lty=par("lty"), root.edge=TRUE, stratified=stratified)

plot_BioGeoBEARS_results(results_object=resmod, analysis_titletxt="Stochastic map", addl_params=list("j"), plotwhat="text", cornercoords_loc=scriptdir, root.edge=TRUE, colors_list_for_states=colors_list_for_states, skiptree=TRUE, show.tip.label=TRUE)

############################################
# Close PDF
############################################
dev.off()
cmdstr = paste("open ", pdffn, sep="")
system(cmdstr)

#######################################################
# Plot all 100 stochastic maps to PDF
#######################################################
# Setup
include_null_range = include_null_range
areanames = areanames
areas = areanames
max_range_size = max_range_size
states_list_0based = rcpp_areas_list_to_states_list(areas=areas, maxareas=max_range_size, include_null_range=include_null_range)
colors_list_for_states = get_colors_for_states_list_0based(areanames=areanames, states_list_0based=states_list_0based, max_range_size=max_range_size, plot_null_range=TRUE)
scriptdir = np(system.file("extdata/a_scripts", package="BioGeoBEARS"))
stratified = stratified

#Manually inserting color codes again
colors_list_for_states<-c("#000000", "#0000FF", "#00FFFF", "#00CD00", "#FFFF00", "#FF0000", "#0080FF", "#006680", "#808080",
                          "#800080", "#00E680", "#80FF80", "#808080", "#80E600", "#806600", "#FF8000", "#0099AA", 
                          "#55AAAA", "#5555AA", "#559955", "#554455", "#AA5555", "#55EE55", "#559955", "#AAAA55",
                          "#AA9900")

# Loop through the maps and plot to PDF
pdffn = paste0(model_name, "_", length(clado_events_tables), "BSMs_DEC_M2.pdf")
pdf(file=pdffn, width=6, height=6)

nummaps_goal = 100
for (i in 1:nummaps_goal)
{
  clado_events_table = clado_events_tables[[i]]
  analysis_titletxt = paste0(model_name, " - Stochastic Map #", i, "/", nummaps_goal)
  plot_BSM(results_object=res, clado_events_table=clado_events_table, stratified=stratified, analysis_titletxt=analysis_titletxt, addl_params=list("j"), label.offset=0.5, plotwhat="text", cornercoords_loc=scriptdir, root.edge=TRUE, colors_list_for_states=colors_list_for_states, show.tip.label=TRUE, include_null_range=include_null_range)
} # END for (i in 1:nummaps_goal)

dev.off()
cmdstr = paste("open ", pdffn, sep="")
system(cmdstr)

#######################################################
# Summarize stochastic map tables
#######################################################
length(clado_events_tables)
length(ana_events_tables)

head(clado_events_tables[[1]][,-20])
tail(clado_events_tables[[1]][,-20])

head(ana_events_tables[[1]])
tail(ana_events_tables[[1]])

areanames = names(tipranges@df)
actual_names = areanames
actual_names

# Get the dmat and times (if any)
dmat_times = get_dmat_times_from_res(res=res, numstates=NULL)
dmat_times

# Extract BSM output
clado_events_tables = BSM_output$RES_clado_events_tables
ana_events_tables = BSM_output$RES_ana_events_tables

# Simulate the source areas
BSMs_w_sourceAreas = simulate_source_areas_ana_clado(res, clado_events_tables, ana_events_tables, areanames)
clado_events_tables = BSMs_w_sourceAreas$clado_events_tables
ana_events_tables = BSMs_w_sourceAreas$ana_events_tables

# Count all anagenetic and cladogenetic events
counts_list = count_ana_clado_events(clado_events_tables, ana_events_tables, areanames, actual_names)

summary_counts_BSMs = counts_list$summary_counts_BSMs
print(conditional_format_table(summary_counts_BSMs))

# Histogram of event counts
hist_event_counts(counts_list, pdffn=paste0(model_name, "_histograms_of_event_counts.pdf"))

#######################################################
# Print counts to files
#######################################################
tmpnames = names(counts_list)
cat("\n\nWriting tables* of counts to tab-delimited text files:\n(* = Tables have dimension=2 (rows and columns). Cubes (dimension 3) and lists (dimension 1) will not be printed to text files.) \n\n")
for (i in 1:length(tmpnames))
{
  cmdtxt = paste0("item = counts_list$", tmpnames[i])
  eval(parse(text=cmdtxt))
  
  # Skip cubes
  if (length(dim(item)) != 2)
  {
    next()
  }
  
  outfn = paste0(tmpnames[i], ".txt")
  if (length(item) == 0)
  {
    cat(outfn, " -- NOT written, *NO* events recorded of this type", sep="")
    cat("\n")
  } else {
    cat(outfn)
    cat("\n")
    write.table(conditional_format_table(item), file=outfn, quote=FALSE, sep="\t", col.names=TRUE, row.names=TRUE)
  } # END if (length(item) == 0)
} # END for (i in 1:length(tmpnames))
cat("...done.\n")

#######################################################
# Check that ML ancestral state/range probabilities and
# the mean of the BSMs approximately line up
#######################################################
library(MultinomialCI)    # For 95% CIs on BSM counts
check_ML_vs_BSM(res, clado_events_tables, model_name, tr=NULL, plot_each_node=FALSE, linreg_plot=TRUE, MultinomialCI=TRUE)
